@@ -1,22 +1,22 @@
 use strict;
 use warnings;
-package JSON::Schema::Modern; # git description: v0.522-7-g205a42ce
+package JSON::Schema::Modern; # git description: v0.523-21-g52577441
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate data against a schema
 # KEYWORDS: JSON Schema data validation structure specification
 
-our $VERSION = '0.523';
+our $VERSION = '0.524';
 
 use 5.016;  # for fc, unicode_strings features
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
-use if "$]" >= 5.022, 'experimental', 're_strict';
+use if "$]" >= 5.022, experimental => 're_strict';
 use strictures 2;
 use JSON::MaybeXS;
 use Carp qw(croak carp);
 use List::Util 1.55 qw(pairs first uniqint pairmap);
-use Ref::Util 0.100 qw(is_ref is_hashref);
+use Ref::Util 0.100 qw(is_ref is_plain_hashref);
 use Mojo::URL;
 use Safe::Isa;
 use Path::Tiny;
@@ -134,7 +134,7 @@ sub add_schema {
     : JSON::Schema::Modern::Document->new(
       schema => shift,
       $uri ? (canonical_uri => $uri) : (),
-      _evaluator => $self,  # used only for traversal during document construction
+      evaluator => $self,  # used mainly for traversal during document construction
     );
 
   croak(!(caller())[0]->isa(__PACKAGE__)
@@ -223,8 +223,6 @@ sub traverse {
     initial_schema_uri => $base_uri,    # the canonical URI as of the start, or last $id
     schema_path => '',                  # the rest of the path, since the last $id
     errors => [],
-    spec_version => $spec_version,
-    vocabularies => [ use_module('JSON::Schema::Modern::Vocabulary::Core') ], # will be filled in later
     identifiers => [],
     configs => {},
     callbacks => $config_override->{callbacks} // {},
@@ -232,17 +230,26 @@ sub traverse {
   };
 
   try {
-    $self->_traverse_subschema(
-      is_hashref($schema_reference) && !(exists $schema_reference->{'$schema'})
-        ? +{
-          # ensure that specification version and vocabularies are properly determined
-          '$schema' => $self->METASCHEMA_URIS->{$spec_version},
-          %$schema_reference,
-        }
-        : $schema_reference,
-      $state,
+    my $for_canonical_uri = Mojo::URL->new(
+      (is_plain_hashref($schema_reference) && exists $schema_reference->{'$id'}
+          ? Mojo::URL->new($schema_reference->{'$id'}) : undef)
+        // $state->{initial_schema_uri});
+    $for_canonical_uri->fragment(undef) if not length $for_canonical_uri->fragment;
+
+    # a subsequent "$schema" keyword can still change these values
+    @{$state}{qw(spec_version vocabularies)} = $self->_get_metaschema_info(
+      $config_override->{metaschema_uri} // $self->METASCHEMA_URIS->{$spec_version},
+      $for_canonical_uri,
     );
   }
+  catch ($e) {
+    push @{$state->{errors}}, $e->errors;
+    return $state;
+  }
+
+  try {
+    $self->_traverse_subschema($schema_reference, $state);
+  }
   catch ($e) {
     if ($e->$_isa('JSON::Schema::Modern::Error')) {
       # note: we should never be here, since traversal subs are no longer be fatal
@@ -265,7 +272,7 @@ sub evaluate {
   my $base_uri = Mojo::URL->new;  # TODO: will be set by a global attribute
 
   my $state = {
-    data_path => '',
+    data_path => $config_override->{data_path} // '',
     traversed_schema_path => '',        # the accumulated traversal path as of the start, or last $id, or up to the last traversed $ref
     initial_schema_uri => $base_uri,    # the canonical URI as of the start or last $id, or the last traversed $ref
     schema_path => '',                  # the rest of the path, since the last $id or the last traversed $ref
@@ -530,6 +537,7 @@ has _resource_index => (
     _resource_keys => 'keys',
     _add_resources_unsafe => 'set',
     _resource_values => 'values',
+    _resource_exists => 'exists',
   },
   lazy => 1,
   default => sub { {} },
@@ -582,6 +590,7 @@ has _vocabulary_classes => (
   handles => {
     _get_vocabulary_class => 'get',
     _set_vocabulary_class => 'set',
+    _get_vocabulary_values => 'values',
   },
   lazy => 1,
   default => sub {
@@ -596,6 +605,8 @@ has _vocabulary_classes => (
 sub add_vocabulary {
   my ($self, $classname) = @_;
 
+  return if grep $_->[1] eq $classname, $self->_get_vocabulary_values;
+
   $vocabulary_class_type->(use_module($classname));
 
   # uri => version, uri => version
@@ -635,6 +646,39 @@ has _metaschema_vocabulary_classes => (
   },
 );
 
+# retrieves metaschema info either from cache or by parsing the schema for vocabularies
+# throws a JSON::Schema::Modern::Result on error
+sub _get_metaschema_info {
+  my ($self, $metaschema_uri, $for_canonical_uri) = @_;
+
+  # check the cache
+  my $metaschema_info = $self->_get_metaschema_vocabulary_classes($metaschema_uri);
+  return @$metaschema_info if $metaschema_info;
+
+  # otherwise, fetch the metaschema and parse its $vocabulary keyword.
+  # we do this by traversing a baby schema with just the $schema keyword.
+  my $state = $self->traverse({ '$schema' => $metaschema_uri.'' });
+  die JSON::Schema::Modern::Result->new(
+    output_format => $self->output_format,
+    valid => JSON::PP::false,
+    errors => [
+      map {
+        my $e = $_;
+        # absolute location is undef iff the location = '/$schema'
+        my $absolute_location = $e->absolute_keyword_location // $for_canonical_uri;
+        JSON::Schema::Modern::Error->new(
+          keyword => $e->keyword eq '$schema' ? '' : $e->keyword,
+          instance_location => $e->instance_location,
+          keyword_location => ($for_canonical_uri->fragment//'').($e->keyword_location =~ s{^/\$schema\b}{}r),
+          length $absolute_location ? ( absolute_keyword_location => $absolute_location ) : (),
+          error => $e->error,
+        )
+      }
+      @{$state->{errors}} ],
+  ) if @{$state->{errors}};
+  return ($state->{spec_version}, $state->{vocabularies});
+}
+
 # used for determining a default '$schema' keyword where there is none
 use constant METASCHEMA_URIS => {
   'draft2020-12' => 'https://json-schema.org/draft/2020-12/schema',
@@ -643,14 +687,19 @@ use constant METASCHEMA_URIS => {
 };
 
 use constant CACHED_METASCHEMAS => {
+  'https://json-schema.org/draft/2020-12/hyper-schema'        => 'draft2020-12/hyper-schema.json',
+  'https://json-schema.org/draft/2020-12/links'               => 'draft2020-12/links.json',
   'https://json-schema.org/draft/2020-12/meta/applicator'     => 'draft2020-12/meta/applicator.json',
   'https://json-schema.org/draft/2020-12/meta/content'        => 'draft2020-12/meta/content.json',
   'https://json-schema.org/draft/2020-12/meta/core'           => 'draft2020-12/meta/core.json',
   'https://json-schema.org/draft/2020-12/meta/format-annotation' => 'draft2020-12/meta/format-annotation.json',
   'https://json-schema.org/draft/2020-12/meta/format-assertion'  => 'draft2020-12/meta/format-assertion.json',
+  'https://json-schema.org/draft/2020-12/meta/hyper-schema'   => 'draft2020-12/meta/hyper-schema.json',
   'https://json-schema.org/draft/2020-12/meta/meta-data'      => 'draft2020-12/meta/meta-data.json',
   'https://json-schema.org/draft/2020-12/meta/unevaluated'    => 'draft2020-12/meta/unevaluated.json',
   'https://json-schema.org/draft/2020-12/meta/validation'     => 'draft2020-12/meta/validation.json',
+  #'https://json-schema.org/draft/2020-12/output/hyper-schema' => 'draft2020-12/output/hyper-schema.json',
+  'https://json-schema.org/draft/2020-12/output/schema'       => 'draft2020-12/output/schema.json',
   'https://json-schema.org/draft/2020-12/schema'              => 'draft2020-12/schema.json',
 
   'https://json-schema.org/draft/2019-09/hyper-schema'        => 'draft2019-09/hyper-schema.json',
@@ -680,7 +729,7 @@ sub _get_or_load_resource {
   if (my $local_filename = $self->CACHED_METASCHEMAS->{$uri}) {
     my $file = path(dist_dir('JSON-Schema-Modern'), $local_filename);
     my $schema = $self->_json_decoder->decode($file->slurp_raw);
-    my $document = JSON::Schema::Modern::Document->new(schema => $schema, _evaluator => $self);
+    my $document = JSON::Schema::Modern::Document->new(schema => $schema, evaluator => $self);
 
     # this should be caught by the try/catch in evaluate()
     die JSON::Schema::Modern::Result->new(
@@ -710,6 +759,8 @@ sub _get_or_load_resource {
 # - the canonical uri for that schema,
 # - the JSON::Schema::Modern::Document object that holds that schema
 # - the path relative to the document root for this schema
+# - the specification version that applies to this schema
+# - the vocabularies to use when considering schema keywords
 # creates a Document and adds it to the resource index, if not already present.
 sub _fetch_from_uri {
   my ($self, $uri) = @_;
@@ -782,7 +833,7 @@ JSON::Schema::Modern - Validate data against a schema
 
 =head1 VERSION
 
-version 0.523
+version 0.524
 
 =head1 SYNOPSIS
 
@@ -1041,9 +1092,12 @@ For more information, see L<Cpanel::JSON::XS/MAPPING>.
 
 =head2 Format Validation
 
-By default, formats are treated only as annotations, not assertions. When L</validate_formats> is
+By default (and unless you specify a custom metaschema with the C<$schema> keyword or
+L<JSON::Schema::Modern::Document/metaschema>),
+formats are treated only as annotations, not assertions. When L</validate_formats> is
 true, strings are also checked against the format as specified in the schema. At present the
-following formats are supported (use of any other formats than these will always evaluate as true):
+following formats are supported (use of any other formats than these will always evaluate as true,
+but remember you can always supply custom format handlers; see L</format_validations> above):
 
 =over 4
 
