@@ -4,17 +4,18 @@ package JSON::Schema::Modern::Error;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Contains a single error from a JSON Schema evaluation
 
-our $VERSION = '0.523';
+our $VERSION = '0.524';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
-use if "$]" >= 5.022, 'experimental', 're_strict';
+use if "$]" >= 5.022, experimental => 're_strict';
 use strictures 2;
+use Safe::Isa;
 use Moo;
 use MooX::TypeTiny;
-use Types::Standard qw(Str Undef);
+use Types::Standard qw(Str Undef InstanceOf);
 use namespace::clean;
 
 has [qw(
@@ -29,8 +30,8 @@ has [qw(
 
 has absolute_keyword_location => (
   is => 'ro',
-  isa => Str, # always a uri (absolute uri or uri reference)
-  coerce => sub { "$_[0]" },
+  isa => InstanceOf['Mojo::URL'],
+  coerce => sub { $_[0]->$_isa('Mojo::URL') ? $_[0] : Mojo::URL->new($_[0]) },
 );
 
 has keyword => (
@@ -46,7 +47,7 @@ sub TO_JSON {
     instanceLocation => $self->instance_location,
     keywordLocation => $self->keyword_location,
     !defined($self->absolute_keyword_location) ? ()
-      : ( absoluteKeywordLocation => $self->absolute_keyword_location ),
+      : ( absoluteKeywordLocation => $self->absolute_keyword_location->to_string ),
     error => $self->error,  # TODO: allow localization
   };
 }
@@ -67,7 +68,7 @@ JSON::Schema::Modern::Error - Contains a single error from a JSON Schema evaluat
 
 =head1 VERSION
 
-version 0.523
+version 0.524
 
 =head1 SYNOPSIS
 
