@@ -4,13 +4,13 @@ package JSON::Schema::Modern::Utilities;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Internal utilities for JSON::Schema::Modern
 
-our $VERSION = '0.523';
+our $VERSION = '0.524';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
-use if "$]" >= 5.022, 'experimental', 're_strict';
+use if "$]" >= 5.022, experimental => 're_strict';
 use strictures 2;
 use B;
 use Carp 'croak';
@@ -36,8 +36,10 @@ our @EXPORT_OK = qw(
   E
   A
   abort
+  assert_keyword_exists
   assert_keyword_type
   assert_pattern
+  is_uri_reference
   assert_uri_reference
   assert_uri
   annotate_self
@@ -190,6 +192,14 @@ sub canonical_schema_uri {
 }
 
 # shorthand for creating error objects
+# uses these keys from $state:
+# - initial_schema_uri
+# - keyword
+# - data_path
+# - traversed_schema_path
+# - schema_path
+# - _schema_path_suffix
+# - errors
 sub E {
   croak 'E called in void context' if not defined wantarray;
   my ($state, $error_string, @args) = @_;
@@ -215,6 +225,15 @@ sub E {
 }
 
 # shorthand for creating annotations
+# uses these keys from $state:
+# - initial_schema_uri
+# - keyword
+# - data_path
+# - traversed_schema_path
+# - schema_path
+# - _schema_path_suffix
+# - annotations
+# - collect_annotations
 sub A {
   my ($state, $annotation) = @_;
   return 1 if not $state->{collect_annotations};
@@ -248,6 +267,13 @@ sub abort {
   die pop @{$state->{errors}};
 }
 
+sub assert_keyword_exists {
+  croak 'assert_keyword_exists called in void context' if not defined wantarray;
+  my ($state, $schema) = @_;
+  return E($state, '%s keyword is required', $state->{keyword}) if not exists $schema->{$state->{keyword}};
+  return 1;
+}
+
 sub assert_keyword_type {
   croak 'assert_keyword_type called in void context' if not defined wantarray;
   my ($state, $schema, $type) = @_;
@@ -271,13 +297,11 @@ sub assert_pattern {
   return 1;
 }
 
-sub assert_uri_reference {
-  croak 'assert_uri_reference called in void context' if not defined wantarray;
-  my ($state, $schema) = @_;
+sub is_uri_reference {
+  croak 'is_uri_reference called in void context' if not defined wantarray;
+  my $ref = shift;
 
-  my $ref = $schema->{$state->{keyword}};
-
-  return E($state, '%s value is not a valid URI reference', $state->{keyword})
+  return 0
     # see also uri-reference format sub
     if fc(Mojo::URL->new($ref)->to_unsafe_string) ne fc($ref)
       or $ref =~ /[^[:ascii:]]/
@@ -289,6 +313,14 @@ sub assert_uri_reference {
   return 1;
 }
 
+sub assert_uri_reference {
+  croak 'assert_uri_reference called in void context' if not defined wantarray;
+  my ($state, $schema) = @_;
+
+  return 1 if is_uri_reference($schema->{$state->{keyword}});
+  return E($state, '%s value is not a valid URI reference', $state->{keyword});
+}
+
 sub assert_uri {
   croak 'assert_uri called in void context' if not defined wantarray;
   my ($state, $schema, $override) = @_;
@@ -330,7 +362,7 @@ JSON::Schema::Modern::Utilities - Internal utilities for JSON::Schema::Modern
 
 =head1 VERSION
 
-version 0.523
+version 0.524
 
 =head1 SYNOPSIS
 
@@ -341,8 +373,8 @@ version 0.523
 This class contains internal utilities to be used by L<JSON::Schema::Modern>.
 
 =for Pod::Coverage is_type get_type is_equal is_elements_unique jsonp local_annotations
-canonical_schema_uri E A abort assert_keyword_type assert_pattern assert_uri_reference assert_uri
-annotate_self
+canonical_schema_uri E A abort assert_keyword_exists assert_keyword_type assert_pattern assert_uri_reference assert_uri
+annotate_self is_uri_reference
 
 =head1 SUPPORT
 
