@@ -4,13 +4,13 @@ package JSON::Schema::Modern::Vocabulary::FormatAssertion;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Format-Assertion vocabulary
 
-our $VERSION = '0.523';
+our $VERSION = '0.524';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
-use if "$]" >= 5.022, 'experimental', 're_strict';
+use if "$]" >= 5.022, experimental => 're_strict';
 use strictures 2;
 use JSON::Schema::Modern::Utilities qw(is_type E A assert_keyword_type);
 use Moo;
@@ -202,7 +202,7 @@ JSON::Schema::Modern::Vocabulary::FormatAssertion - Implementation of the JSON S
 
 =head1 VERSION
 
-version 0.523
+version 0.524
 
 =head1 DESCRIPTION
 
